/* Copyright (C) 2002-2004  The Coaster Development Team
 *               
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <config.h>
#include <string.h>
#include <gnome.h>
#include <gconf/gconf-client.h>
#include <libburn/libburn.h>

#include "coaster.h"
#include "common.h"

#define SIZE 256

FILE *log_file;

GConfClient *coaster_client;

gboolean
coaster_init (gpointer* drives_pointer, int *n_drives, gboolean debug_on)
{
  struct burn_drive_info* drives;
  int i, num_drives;

  coaster_client = gconf_client_get_default();

  /* Check for debugging */
  if (debug_on) debug = TRUE;
  else debug = FALSE;

  if (debug) {
    g_print ("libcoaster startup\n");
    
    g_print("Initializing libburn...");
  }
  
  /* Init libburn library */
  if (burn_initialize()) {
  	if (debug) g_print("Success\n\n");
  } else {
  	if (debug) g_print("Failed\n\n");
    return FALSE;
  }

  if (debug) g_print ("Scanning for devices...");
  while (!burn_drive_scan (&drives, n_drives));
  if (debug) {
    g_print ("Done\n");
    g_print ("Devices (%u found)\n", *n_drives);
  }

  if (debug) {
    for (i=0; i < *n_drives; i++) { 
  	  char *name = g_strconcat (drives[i].location, ": ", drives[i].vendor,
  		  					  " - ", drives[i].product, NULL);
      g_print ("%s\n", name);
      g_free (name);
    }
  }

  if (debug) g_print ("libcoaster startup successful\n");

  *drives_pointer = drives;

  return TRUE;
}

gboolean
coaster_refresh_drives ()
{
  burn_drive_info_free (drives);
  if (debug) g_print ("libcoaster refreshing drives...\n");
  while(!burn_drive_scan (&drives, &n_drives));
  if (debug) g_print ("libcoaster finished refreshing drives...\n");

  return TRUE;
}

char*
coaster_get_read_drive_location (GError *error, GCallback *on_update)
{
  GConfClient *gconf_client;
  char *return_val;
  
  gconf_client = coaster_get_gconf_client ();
  return_val = gconf_client_get_string (gconf_client, OPTICAL_READER_KEY, NULL);
  
  return return_val;
}

char*
coaster_get_write_drive_location (GError *error, GCallback *on_update)
{
  GConfClient *gconf_client;
  char *return_val;
  
  gconf_client = coaster_get_gconf_client ();
  return_val = gconf_client_get_string (gconf_client, OPTICAL_WRITER_KEY, NULL);
  
  return return_val;
}

int
coaster_get_read_speed (GError *error, GCallback *on_update)
{
  GConfClient *gconf_client;
  int return_val;
  
  gconf_client = coaster_get_gconf_client ();
  return_val = gconf_client_get_int (gconf_client, READ_SPEED_KEY, NULL);
  
  return return_val;
}

int
coaster_get_write_speed (GError *error, GCallback *on_update)
{
  GConfClient *gconf_client;
  int return_val;
  
  gconf_client = coaster_get_gconf_client ();
  return_val = gconf_client_get_int (gconf_client, WRITE_SPEED_KEY, NULL);
  
  return return_val;
}

gboolean
coaster_get_eject (GError *error, GCallback *on_update)
{
  GConfClient *gconf_client;
  gboolean return_val;
  
  gconf_client = coaster_get_gconf_client ();
  return_val = gconf_client_get_bool (gconf_client, EJECT_KEY, NULL);
  
  return return_val;
}

gboolean
coaster_finish () {
  if (debug) g_print ("libcoaster shutting down...\n");
  burn_drive_info_free (drives);
  burn_finish ();
  g_object_unref (coaster_client);
  
  return TRUE;
}

GConfClient*
coaster_get_gconf_client()
{
  return coaster_client;
}

gboolean
coaster_verify_read_drive_location(gpointer drives, char *location)
{
  /* Check to make sure that the device exists*/
  /* Check to make sure that the device is detect by libburn */
  /* Check that it is a reader */
  return TRUE;
}

gboolean
coaster_verify_write_drive_location(gpointer drives, char *location)
{
  /* Check to make sure that the device exists*/
  /* Check to make sure that the device is detect by libburn */
  /* Check that is is a writer*/
  return TRUE;
}

gboolean
coaster_verify_speed(gpointer drive, int speed)
{
  /* Check to make sure that the speed is a valid speed for the drive*/
  return TRUE;
}
