/* Copyright (C) 2002-2004  The Coaster Development Team
 *               
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <config.h>
#include <string.h>
#include <gnome.h>
#include <gtk/gtk.h>
#include <gtk/gtkwindow.h>
#include <gconf/gconf-client.h>
#include <libburn/libburn.h>

#include "coaster.h"
#include "common.h"
#include "preferences.h"

/* Functions to create the user interface */
GtkWidget*
coaster_preferences_window (gpointer drives_pointer, int n_drives_passed, GCallback *on_close_function)
{
  GConfClient *gconf_client;
 
  drives = (struct burn_drive_info*) drives_pointer;
  n_drives = n_drives_passed;

  gconf_client = coaster_get_gconf_client ();   
  gconf_client_notify_add (gconf_client, "/desktop/gnome/burn_prefs/optical_reader_dev", on_gconf_ord_change, NULL, NULL, NULL);
  gconf_client_notify_add (gconf_client, "/desktop/gnome/burn_prefs/optical_writer_dev", on_gconf_owd_change, NULL, NULL, NULL);
  gconf_client_notify_add (gconf_client, "/desktop/gnome/burn_prefs/write_speed", on_gconf_write_speed_change, NULL, NULL, NULL);
  gconf_client_notify_add (gconf_client, "/desktop/gnome/burn_prefs/eject", on_gconf_eject_change, NULL, NULL, NULL);
  
  return draw_window(on_close_function);
}

GtkWidget*
draw_window (GCallback* on_close_function)
{
  GtkWidget *window;
  char *read_drive_location, *write_drive_location;
  int write_speed;
  gboolean eject_toggle;

  window = create_preferences_window (on_close_function);

  /* Set the default values*/
  read_drive_location = coaster_get_read_drive_location (NULL, NULL);
  write_drive_location = coaster_get_write_drive_location (NULL, NULL);
  write_speed = coaster_get_write_speed (NULL, NULL);
  eject_toggle = coaster_get_eject (NULL, NULL);
  
  /* FIXME: Need to write in fail code so it acts gracefully */
  // coaster_verify_read_drive_location ((gpointer)drives, read_drive_location);
  // coaster_verify_write_drive_location ((gpointer)drives, write_drive_location);
  // coaster_verify_speed (NULL, write_speed);
  
  set_active_reader (read_drive_location);
  set_active_writer (write_drive_location);
  set_write_speed (write_speed);
  set_eject_disc (eject_toggle);

  //gtk_widget_show (window); 
  return window;
}

GtkWidget*
create_preferences_window (GCallback* on_close_function)
{
  GtkWidget *preferences_window;
  GtkWidget *dialog_vbox;
  GtkWidget *vbox1;
  GtkWidget *vbox2;
  GtkWidget *odr_label;
  GtkWidget *hbox1;
  GtkWidget *orp_label;
  GtkWidget *menu1;
  GtkWidget *vbox3;
  GtkWidget *owd_label;
  GtkWidget *hbox2;
  GtkWidget *owp_label;
  GtkWidget *hbox3;
  GtkWidget *ows_label;
  GtkWidget *vbox4;
  GtkWidget *odo_label;
  GtkWidget *hbox4;
  GtkWidget *indent;
  GtkWidget *vbox5;
  GtkWidget *dialog_action_area;
  GtkWidget *close_button;

  // Need to make sure the layout is HIG compatible
  preferences_window = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (preferences_window), "Optical Device Preferences");
  gtk_window_set_resizable (GTK_WINDOW (preferences_window), FALSE);
  gtk_dialog_set_has_separator (GTK_DIALOG (preferences_window), FALSE);

  dialog_vbox = GTK_DIALOG (preferences_window)->vbox;
  gtk_widget_show (dialog_vbox);
  gtk_widget_set_size_request (dialog_vbox, 410, -1);

  vbox1 = gtk_vbox_new (FALSE, 18);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox), vbox1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (vbox1), 12);

  vbox2 = gtk_vbox_new (FALSE, 6);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), vbox2, TRUE, TRUE, 0);

  odr_label = gtk_label_new ("<b>Optical Reader Device</b>");
  gtk_widget_show (odr_label);
  gtk_box_pack_start (GTK_BOX (vbox2), odr_label, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (odr_label), TRUE);
  gtk_misc_set_alignment (GTK_MISC (odr_label), 0, 0.5);

  hbox1 = gtk_hbox_new (FALSE, 12);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox1, TRUE, TRUE, 0);

  orp_label = gtk_label_new_with_mnemonic ("    Optical _reader path:");
  gtk_widget_show (orp_label);
  gtk_box_pack_start (GTK_BOX (hbox1), orp_label, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (orp_label), 0, 0.5);

  readers = populate_readers ();
  gtk_box_pack_start (GTK_BOX (hbox1), readers, FALSE, FALSE, 0);

  vbox3 = gtk_vbox_new (FALSE, 6);
  gtk_widget_show (vbox3);
  gtk_box_pack_start (GTK_BOX (vbox1), vbox3, TRUE, TRUE, 0);

  owd_label = gtk_label_new ("<b>Optical Writer Device</b>");
  gtk_widget_show (owd_label);
  gtk_box_pack_start (GTK_BOX (vbox3), owd_label, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (owd_label), TRUE);
  gtk_misc_set_alignment (GTK_MISC (owd_label), 0, 0.5);

  hbox2 = gtk_hbox_new (FALSE, 12);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox2, TRUE, TRUE, 0);

  owp_label = gtk_label_new_with_mnemonic ("    Optical _writer path:");
  gtk_widget_show (owp_label);
  gtk_box_pack_start (GTK_BOX (hbox2), owp_label, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (owp_label), 0, 0.5);

  writers = populate_writers ();
  gtk_box_pack_start (GTK_BOX (hbox2), writers, FALSE, FALSE, 0);

  hbox3 = gtk_hbox_new (FALSE, 12);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox3, TRUE, TRUE, 0);

  ows_label = gtk_label_new_with_mnemonic ("    Optical writer _speed:");
  gtk_widget_show (ows_label);
  gtk_box_pack_start (GTK_BOX (hbox3), ows_label, FALSE, FALSE, 0);

  w_speed = gtk_combo_box_new ();
  gtk_box_pack_start (GTK_BOX (hbox3), w_speed, FALSE, FALSE, 0);

  vbox4 = gtk_vbox_new (FALSE, 6);
  gtk_widget_show (vbox4);
  gtk_box_pack_start (GTK_BOX (vbox1), vbox4, TRUE, TRUE, 0);

  odo_label = gtk_label_new ("<b>Optical Device Options</b>");
  gtk_widget_show (odo_label);
  gtk_box_pack_start (GTK_BOX (vbox4), odo_label, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (odo_label), TRUE);
  gtk_misc_set_alignment (GTK_MISC (odo_label), 0, 0.5);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox4, TRUE, TRUE, 0);

  indent = gtk_label_new ("    ");
  gtk_widget_show (indent);
  gtk_box_pack_start (GTK_BOX (hbox4), indent, FALSE, FALSE, 0);

  vbox5 = gtk_vbox_new (FALSE, 6);
  gtk_widget_show (vbox5);
  gtk_box_pack_start (GTK_BOX (hbox4), vbox5, TRUE, TRUE, 0);

  eject = gtk_check_button_new_with_mnemonic ("_Eject Disc when done");
  gtk_widget_show (eject);
  gtk_box_pack_start (GTK_BOX (vbox5), eject, FALSE, FALSE, 0);

  dialog_action_area = GTK_DIALOG (preferences_window)->action_area;
  gtk_widget_show (dialog_action_area);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area), GTK_BUTTONBOX_END);

  close_button = gtk_button_new_from_stock ("gtk-close");
  gtk_widget_show (close_button);
  gtk_dialog_add_action_widget (GTK_DIALOG (preferences_window), close_button, GTK_RESPONSE_CLOSE);
  GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) readers, "changed",
                    G_CALLBACK (on_orp_option_changed),
                    NULL);

  g_signal_connect ((gpointer) writers, "changed",
                    G_CALLBACK (on_owp_option_changed),
                    NULL);

  g_signal_connect ((gpointer) w_speed, "changed",
                    G_CALLBACK (on_ows_option_changed),
                    NULL);

  g_signal_connect ((gpointer) eject, "toggled",
                    G_CALLBACK (on_eject_disc_toggled),
                    NULL);

  g_signal_connect ((gpointer) close_button, "clicked",
		    		G_CALLBACK (on_close_function),
		    		(gpointer)(preferences_window));

  g_signal_connect ((gpointer) preferences_window, "destroy",
		    		G_CALLBACK (on_close_function), 
		    		NULL);

  gtk_label_set_mnemonic_widget (GTK_LABEL (orp_label), readers);
  gtk_label_set_mnemonic_widget (GTK_LABEL (owp_label), writers);
  gtk_label_set_mnemonic_widget (GTK_LABEL (ows_label), w_speed);

  return preferences_window;
}

GtkWidget*
populate_readers ()
{
  unsigned int i = 0;
  GtkWidget *combo;
  GtkListStore *drives_store;
  GtkTreeIter iter;
  GtkCellRenderer *renderer;

  drives_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_POINTER);

  for (i=0; i < n_drives; i++) { 
  	if (drives[i].read_cdr) {
		char *name = g_strconcat (drives[i].vendor, " - ", drives[i].product, NULL);
		gtk_list_store_append (drives_store, &iter);
		gtk_list_store_set (drives_store, &iter, 0, name, 1, &drives[i], -1);
		g_free (name);
    }
  }

  combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (drives_store));

  gtk_cell_layout_clear (GTK_CELL_LAYOUT (combo));
  renderer = gtk_cell_renderer_text_new ();
  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), renderer,
				  				  "text", 0, NULL);
  gtk_widget_show (combo);
  
  return combo;
}

GtkWidget*
populate_writers ()
{
  unsigned int i = 0;
  GtkWidget *combo;
  GtkListStore *drives_store;
  GtkTreeIter iter;
  GtkCellRenderer *renderer;

  drives_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_POINTER);

  for (i=0; i < n_drives; i++) { 
  	if (drives[i].write_cdr) {
	  char *name = g_strconcat (drives[i].vendor, " - ", drives[i].product, NULL);
	  gtk_list_store_append (drives_store, &iter);
	  gtk_list_store_set (drives_store, &iter, 0, name, 1, &drives[i], -1);
	  g_free (name);
	}
  }

  combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (drives_store));

  gtk_cell_layout_clear (GTK_CELL_LAYOUT (combo));
  renderer = gtk_cell_renderer_text_new ();
  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), renderer,
				  				  "text", 0, NULL);
  gtk_widget_show (combo);
  
  return combo;
}

void
populate_write_speed ()
{
  int i = 0, speed = 0;
  char *write_drive_location;
  struct burn_drive_info* write_drive_info;
  GtkListStore *speeds_store;
  GtkTreeIter iter;
  GtkCellRenderer *renderer;

  speeds_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_INT);
  write_drive_info = get_active_writer ();
  speed = (burn_drive_get_write_speed (write_drive_info->drive) / 176);

  gtk_list_store_append (speeds_store, &iter);
  gtk_list_store_set (speeds_store, &iter, 0, "Maximum", 1, 0, -1);
  
  for (i=speed; i > 0; i-=2) {
  	char *speed = g_strdup_printf ("%i X", i);
  	gtk_list_store_append (speeds_store, &iter);
  	gtk_list_store_set (speeds_store, &iter, 0, speed, 1, i, -1);
  	g_free (speed);
  }
  
  gtk_combo_box_set_model (GTK_COMBO_BOX (w_speed), 
  						   GTK_TREE_MODEL (speeds_store));
  						   
  gtk_cell_layout_clear (GTK_CELL_LAYOUT (w_speed));
  renderer = gtk_cell_renderer_text_new ();
  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (w_speed), renderer, TRUE);
  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (w_speed), renderer,
  								  "text", 0, NULL);

  gtk_widget_show (w_speed);
}

/* Functions to query the user interface */
struct burn_drive_info*
get_active_reader ()
{
  struct burn_drive_info *active_reader;
  gchar *name;
  GtkTreeIter iter;
  GtkTreeModel *model;
  
  gtk_combo_box_get_active_iter (GTK_COMBO_BOX (readers), &iter);

  model = gtk_combo_box_get_model(GTK_COMBO_BOX (readers));
  gtk_tree_model_get(GTK_TREE_MODEL (model), &iter, 0, &name, 1, &active_reader, -1);

  if (debug) g_print ("\nActive Reader: %s\n", name);

  return active_reader;
}

struct burn_drive_info *
get_active_writer ()
{
  struct burn_drive_info *active_writer;
  gchar *name;
  GtkTreeIter iter;
  GtkTreeModel *model;
  
  gtk_combo_box_get_active_iter (GTK_COMBO_BOX (writers), &iter);

  model = gtk_combo_box_get_model(GTK_COMBO_BOX (writers));
  gtk_tree_model_get(GTK_TREE_MODEL (model), &iter, 0, &name, 1, &active_writer, -1);

  if (debug) g_print ("\nActive Writer: %s\n", name);

  return active_writer;
}

int
get_write_speed ()
{
  int s;
  GtkTreeIter iter;
  GtkTreeModel *model;
	
  gtk_combo_box_get_active_iter (GTK_COMBO_BOX (w_speed), &iter);
  
  model = gtk_combo_box_get_model(GTK_COMBO_BOX (w_speed));
  gtk_tree_model_get(GTK_TREE_MODEL (model), &iter, 1, &s, -1);

  return s;
}

gboolean
get_eject_disc ()
{
  return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (eject));
}

/* Functions to update the user interface */
void
set_active_reader (char *location) 
{
  struct burn_drive_info *drive_info;
  gchar *name;
  GtkTreeIter iter;
  GtkTreeModel *model;
  gboolean valid;
  
  model = gtk_combo_box_get_model(GTK_COMBO_BOX (readers));
  
  valid = gtk_tree_model_get_iter_first (model, &iter);

  while (valid) {
  	gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, 0,
					    &name, 1, &drive_info, -1);
	g_free (name);

	if (g_ascii_strcasecmp(drive_info->location, location) == 0){
  		gtk_combo_box_set_active_iter (GTK_COMBO_BOX (readers), &iter);
	}
    valid = gtk_tree_model_iter_next (model, &iter);
  }
}


void
set_active_writer (char *location)
{
  struct burn_drive_info *drive_info;
  gchar *name;
  GtkTreeIter iter;
  GtkTreeModel *model;
  gboolean valid;
  
  model = gtk_combo_box_get_model (GTK_COMBO_BOX (writers));
  
  valid = gtk_tree_model_get_iter_first (model, &iter);

  while (valid) {
  	gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, 0,
       					&name, 1, &drive_info, -1);
  	g_free (name);

    if (g_ascii_strcasecmp (drive_info->location, location) == 0) {
    	gtk_combo_box_set_active_iter (GTK_COMBO_BOX (writers), &iter);
	}
    valid = gtk_tree_model_iter_next (model, &iter);
  }
}

void
set_write_speed (int speed)
{ 
  gint wspeed;
  gchar *name;
  GtkTreeIter iter;
  GtkTreeModel *model;
  gboolean valid;
	
  model = gtk_combo_box_get_model(GTK_COMBO_BOX (w_speed));
  
  valid = gtk_tree_model_get_iter_first (model, &iter);

  while (valid) {
  	gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, 0,
      					  &name, 1, &wspeed, -1);
    
    if (debug) g_print ("Speed from Combo is: %d\n", wspeed);

    g_free (name);

    if (wspeed == speed) {
    	gtk_combo_box_set_active_iter (GTK_COMBO_BOX (w_speed), &iter);
    	valid = FALSE;
    } else {
    	valid = gtk_tree_model_iter_next (model, &iter);
    }
  }
}

void
set_eject_disc (gboolean state)
{
  return gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (eject), state);
}

/* Gconf update functions */
void
on_gconf_ord_change (GConfClient *client,
		             guint        cnxn_id,
                     GConfEntry  *entry,
                     gpointer     user_data)
{
  set_active_reader (gconf_value_get_string (entry->value));
}

void
on_gconf_owd_change (GConfClient *client,
		             guint        cnxn_id,
                     GConfEntry  *entry,
                     gpointer     user_data)
{
  set_active_writer (gconf_value_get_string (entry->value));
}

void
on_gconf_write_speed_change (GConfClient *client,
                             guint        cnxn_id,
                             GConfEntry  *entry,
                             gpointer     user_data)
{
  set_write_speed (gconf_value_get_int (entry->value));
}

void
on_gconf_eject_change (GConfClient *client,
				       guint        cnxn_id,
                       GConfEntry  *entry,
                       gpointer     user_data)
{
  set_eject_disc (gconf_value_get_bool (entry->value));
}

/* Callbacks */
void
on_orp_option_changed (GtkWidget *widget, gpointer data)
{
  struct burn_drive_info *active_reader;
  GConfClient *gconf_client;

  if (debug) g_print ("Location: %s\n", active_reader->location);

  gconf_client = coaster_get_gconf_client ();
  active_reader = get_active_reader ();
  gconf_client_set_string (gconf_client, OPTICAL_READER_KEY, active_reader->location, NULL);
}

void
on_owp_option_changed (GtkWidget *widget, gpointer data)
{ 
  struct burn_drive_info *active_writer;
  GConfClient *gconf_client;
    
  if (debug) g_print ("Location: %s\n", active_writer->location);

  gconf_client = coaster_get_gconf_client ();
  active_writer = get_active_writer ();
  gconf_client_set_string (gconf_client, OPTICAL_WRITER_KEY, active_writer->location, NULL);
  
  populate_write_speed ();
}

void
on_ows_option_changed (GtkWidget *widget, gpointer data)
{
  GConfClient *gconf_client;

  gconf_client = coaster_get_gconf_client ();
  gconf_client_set_int (gconf_client, WRITE_SPEED_KEY, get_write_speed (), NULL);
}

void
on_eject_disc_toggled (GtkWidget *widget, gpointer data)
{
  GConfClient *gconf_client;
  
  gconf_client = coaster_get_gconf_client ();
  gconf_client_set_bool (gconf_client, EJECT_KEY, get_eject_disc (), NULL);
}
